from srai_core.tools_docker import build_docker
from srai_core.tools_docker import release_docker_aws
from srai_core.tools_docker import clone_repository
from srai_core.tools_env import get_string_from_env
from srai_core.command_handler_subprocess import CommandHandlerSubprocess
import os


def main():
    print("release_docker_aws_local")


if __name__ == "__main__":
    main()
